package com.bwie.service.impl;

import com.alibaba.fastjson.JSON;
import com.alibaba.nacos.common.utils.UuidUtils;
import com.bwie.common.constants.JwtConstants;
import com.bwie.common.constants.TokenConstants;
import com.bwie.common.domain.User;
import com.bwie.common.domain.request.LoginRequest;

import com.bwie.common.domain.response.JwtResponse;
import com.bwie.common.exception.auth.AuthException;
import com.bwie.common.result.Result;
import com.bwie.common.utils.JwtUtils;
import com.bwie.feign.AuthFeign;
import com.bwie.service.AuthService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.stereotype.Service;

import javax.servlet.http.HttpServletRequest;
import java.util.HashMap;
import java.util.concurrent.TimeUnit;


/**
 * @author gxb
 * @description TODO
 * @date 2023-12-08 20:11
 */
@Service
public class AuthServicelmpl implements AuthService {
    @Autowired
    private AuthFeign authFeign;
    @Autowired
    private RedisTemplate<String,String> redisTemplate;
    @Autowired
    private HttpServletRequest request;

    /**
     * 登录
     * @param loginRequest
     * @return
     */
    @Override
    public Result login(LoginRequest loginRequest) {
        /**
         * 判断密码格式
         */
        Boolean userPwd = this.isUserPwd(loginRequest.getPassword());
        if (!userPwd){
            return Result.error("密码由不超过16位的数字、英文、下划线组成");
        }
        /**
         * 判断用户名是否存在
         */
        User user = authFeign.findName(loginRequest.getUsername()).getData();
        if (null==user){
            return Result.error("用户名不存在，请先注册");
        }
        if (!user.getPassword().equals(loginRequest.getPassword())){
            return Result.error("密码输入错误,请重新输入");
        }
        Boolean role = user.getRole().intValue() == loginRequest.getRole();

        if (!role){
            return Result.error("登录身份选择错误");
        }
        JwtResponse token = this.getToken(user);
        if (1==user.getRole()){
            return Result.success(token,"医生登录成功");
        }
        return Result.success(token,"病人（家属）登录成功");
    }

    @Override
    public Result Info() {
        String token = request.getHeader("token");
        String userKey = JwtUtils.getUserKey(token);
        String s = redisTemplate.opsForValue().get(TokenConstants.LOGIN_TOKEN_KEY + userKey);
        User user = JSON.parseObject(s, User.class);
        return Result.success(user,"当前登录人");
    }

    //生成token
    private JwtResponse getToken(User user){
        HashMap<String, Object> map = new HashMap<>();
        String userKey = UuidUtils.generateUuid().replaceAll("-", "");
        map.put(JwtConstants.USER_KEY,userKey);
        String token = JwtUtils.createToken(map);
        redisTemplate.opsForValue().set(TokenConstants.LOGIN_TOKEN_KEY+userKey,
                JSON.toJSONString(user),15, TimeUnit.MINUTES);
        return new JwtResponse(token);
    }

    //编号：必填，且必须是6位数字；
    private Boolean isUserBh(String userBh){
        String str = "^\\d{6}$";
        return userBh.matches(str);
    }
    //密码：必填，由不超过16位的数字、英文、下划线组成
    private Boolean isUserPwd(String userPwd){
        String str = "^[\\w]{1,16}$";
        return userPwd.matches(str);
    }
    //手机号验证
    private Boolean isPhone(String userPhone){
        String str= "^1[3-9]\\d{9}$";
        return userPhone.matches(str);
    }
}
