package com.bwie.es.service.impl;

import cn.hutool.json.ObjectMapper;
import com.bwie.common.domain.Health;
import com.bwie.common.result.PageResult;
import com.bwie.es.domain.request.HealthRequest;
import com.bwie.es.feign.EsFeign;
import com.bwie.es.service.EsService;
import org.elasticsearch.action.search.SearchRequest;
import org.elasticsearch.action.search.SearchResponse;
import org.elasticsearch.client.RequestOptions;
import org.elasticsearch.client.RestHighLevelClient;
import org.elasticsearch.common.text.Text;
import org.elasticsearch.index.query.BoolQueryBuilder;
import org.elasticsearch.index.query.MatchQueryBuilder;
import org.elasticsearch.index.query.QueryBuilders;
import org.elasticsearch.search.SearchHit;
import org.elasticsearch.search.SearchHits;
import org.elasticsearch.search.builder.SearchSourceBuilder;
import org.elasticsearch.search.fetch.subphase.highlight.HighlightBuilder;
import org.elasticsearch.search.fetch.subphase.highlight.HighlightField;
import org.elasticsearch.search.sort.FieldSortBuilder;
import org.elasticsearch.search.sort.SortOrder;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.io.IOException;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;

@Service
public class EsServiceImpl implements EsService{
    @Autowired
    private RestHighLevelClient client;
    @Override
    public PageResult<List<Health>> esList(HealthRequest healthRequest) {
        //
        SearchRequest searchRequest = new SearchRequest("health");
        //
        SearchSourceBuilder sourceBuilder = new SearchSourceBuilder();
        //
        BoolQueryBuilder boolQueryBuilder = QueryBuilders.boolQuery();

        if(healthRequest!=null){
            MatchQueryBuilder healthTitle = QueryBuilders.matchQuery(healthRequest.getHealthTitle(), "healthTitle");
            boolQueryBuilder.must(healthTitle);
        }

        sourceBuilder.sort(new FieldSortBuilder("healthId").order(SortOrder.ASC));//排序

        long total = 0L;
        //高亮设置
        HighlightBuilder highlightBuilder = new HighlightBuilder();

        //设置高属性
        highlightBuilder.field("healthTitle");
        //前缀
        highlightBuilder.preTags("<span style=\"color:red\">");
        //后缀
        highlightBuilder.postTags("</span>");
        sourceBuilder.highlighter(highlightBuilder);

        //绑定请求对象和请求条件
        SearchRequest source = searchRequest.source(sourceBuilder);
        ArrayList<Health> health = new ArrayList<>();
        try {
            SearchResponse search = client.search(source, RequestOptions.DEFAULT);
            SearchHits hits = search.getHits();
            for (SearchHit hit : hits.getHits()) {
                Map<String, HighlightField> highlightFields = hit.getHighlightFields();
                HighlightField name = highlightFields.get("healthTitle");

                StringBuffer stringBuffer = new StringBuffer();
                for (Text fragment : name.getFragments()) {
                    System.out.println("fragment"+fragment);
                    stringBuffer.append(fragment);
                }

                String sourceAsString = hit.getSourceAsString();

            }
        } catch (IOException e) {
            throw new RuntimeException(e);
        }
        return null;
    }
}
