package com.bwie.health.service.impl;

import com.alibaba.fastjson.JSONObject;
import com.bwie.common.constants.TokenConstants;
import com.bwie.common.domain.Health;
import com.bwie.common.domain.User;
import com.bwie.common.exception.auth.AuthException;
import com.bwie.common.result.Result;
import com.bwie.common.utils.JwtUtils;
import com.bwie.health.mapper.HealthMapper;
import com.bwie.health.service.HealthService;
import com.github.pagehelper.PageHelper;
import com.github.pagehelper.PageInfo;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.stereotype.Service;

import javax.servlet.http.HttpServletRequest;
import java.util.List;

@Service
public class HealthServiceImpl implements HealthService{
    @Autowired
    private HealthMapper healthMapper;

    @Autowired
    private RedisTemplate<String,String> redisTemplate;
    @Autowired
    private HttpServletRequest request;

    /*查看内容*/
    @Override
    public Result<PageInfo<Health>> list(Health health) {
        PageHelper.startPage(health.getPageNum(),health.getPageSize());
        List<Health> list = healthMapper.list(health);
        PageInfo<Health> info = new PageInfo<>(list);
        return Result.success(info);
    }

    /*健康教育信息录入*/
    @Override
    public Result<Integer> addHealth(Health health) {
        String token1 = request.getHeader(TokenConstants.TOKEN);
        String userKey = JwtUtils.getUserKey(token1);
        String token = redisTemplate.opsForValue().get(TokenConstants.LOGIN_TOKEN_KEY+userKey);
        User o = JSONObject.parseObject(token, User.class);
        health.setUserId(o.getId());

        Integer i = healthMapper.addHealth(health);

        return Result.success(i,i>0?"录入成功":"录入失败");
    }

    /*健康教育记录修改*/
    @Override
    public Result<Integer> updateHealth(Health health) {
        Integer i = healthMapper.updateHealth(health);
        return Result.success(i,i>0?"修改成功":"修改失败");
    }

    /*健康教育记录删除*/
    @Override
    public Result<Integer> deleteHealth(Integer healthId) {
        Integer i = healthMapper.deleteHealth(healthId);
        return Result.success(i,i>0?"删除成功":"删除失败");
    }
}
