package com.bwie.user.service.impl;

import com.alibaba.fastjson.JSON;
import com.bwie.common.constants.TokenConstants;
import com.bwie.common.domain.Sign;
import com.bwie.common.domain.User;
import com.bwie.common.domain.request.Retri;
import com.bwie.common.result.Result;
import com.bwie.common.utils.JwtUtils;
import com.bwie.user.mapper.UserMapper;
import com.bwie.user.service.UserService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.stereotype.Service;

import javax.servlet.http.HttpServletRequest;
import java.util.List;

/**
 * @author gxb
 * @description TODO
 * @date 2023-12-08 20:06
 */
@Service
public class UserServicelmpl implements UserService {
    @Autowired
    private UserMapper userMapper;
    @Autowired
    private RedisTemplate<String, String> redisTemplate;
    @Autowired
    private HttpServletRequest request;

    /**
     * 用户名查询
     * @param username
     * @return
     */
    @Override
    public Result<User> findName(String username) {
        User user = userMapper.findName(username);
        return Result.success(user);
    }

//    /**
//     * 找回密码
//     *
//     * @param retri
//     * @return
//     */
    @Override
    public Result Retrieve(Retri retri) {
        Boolean phone = this.isPhone(retri.getPhone());
        if (!phone) {
            return Result.error("手机号格式错误");
        }
        User user = userMapper.finduserPhone(retri.getPhone());
        if (null == user) {
            return Result.error("手机号不存在，请注册");
        }
        if (!(retri.getRole() == user.getRole())) {
            return Result.error("身份选择错误,请重新选择身份信息");
        }
        return Result.success("您的密码为："+user.getPassword(), "找回成功");
    }

//    /**
//     * 修改密码
//     *
//     * @param userPwd
//     * @return
//     */
    @Override
    public Result updPwd(String userPwd) {
        Boolean pwd = this.isUserPwd(userPwd);
        if (!pwd) {
            return Result.error("不超过16位的数字、英文、下划线组成");
        }
        //获取当前登录人
        String token = request.getHeader("token");
        String userKey = JwtUtils.getUserKey(token);
        String s = redisTemplate.opsForValue().get(TokenConstants.LOGIN_TOKEN_KEY + userKey);
        User user = JSON.parseObject(s, User.class);
        if (user.getPassword().equals(userPwd)) {
            return Result.error("不能使用旧密码,请重新输入");
        }
        userMapper.updPwds(user.getId(), userPwd);
        return Result.success(200, "修改成功");
    }

//    /**
//     * 注册
//     *
//     * @param sign
//     * @return
//     */
    @Override
    public Result sign(Sign sign) {
        Boolean phone = this.isPhone(sign.getPhone());
        if (!phone) {
            return Result.error("手机号格式错误");
        }

        Boolean userPwd = this.isUserPwd(sign.getPassword());
        if (!userPwd) {
            return Result.error("密码由不超过16位的数字、英文、下划线组成");
        }
        User user = userMapper.finduserPhone(sign.getPhone());
        if (null != user) {
            return Result.error("账号已存在，请前往登录");
        }
        userMapper.Inter(sign);
        return Result.success(200,"注册成功");
    }

//    /**
//     * 添加管理员
//     * @return
//     */
    @Override
    public Result Irole() {
        String token = request.getHeader("token");
        String userKey = JwtUtils.getUserKey(token);
        String s = redisTemplate.opsForValue().get(TokenConstants.LOGIN_TOKEN_KEY + userKey);
        User user = JSON.parseObject(s, User.class);
        if (1==user.getRole()){
            return Result.success(200,"您已经是管理员了，无需升级");
        }
        userMapper.UpdRoles(user.getId());
        return Result.success(200,"恭喜您升级成管理员");
    }
//
//    /**
//     * 管理员管理
//     * @return
//     */
    @Override
    public Result Role() {
        List<User> userList = userMapper.userList(1);
        return Result.success(userList,"管理员列表");
    }



    //密码：必填，由不超过16位的数字、英文、下划线组成
    private Boolean isUserPwd(String userPwd) {
        String str = "^[\\w]{1,16}$";
        return userPwd.matches(str);
    }

    //手机号验证
    private Boolean isPhone(String userPhone) {
        String str = "^1[3-9]\\d{9}$";
        return userPhone.matches(str);
    }

}